#!/usr/bin/env python
"""
This module can be used to regenerate the ``django_countries.countries``
module.

To regenerate that module to contain the latest list of ISO 3166-1 countries,
either call this module directly from the command line
(``python regenenerate.py``), or call the ``regenerate`` method.

"""

import os
import re
import string
import urllib2
from titlecase import titlecase

TEMPLATE = u'''from django.utils.translation import ugettext_lazy as _

# Nicely titled (and translatable) country names.
COUNTRIES = (
%(countries)s
)

# Nicely titled country names with duplicates for those which contain a comma
# (containing the non-comma'd version).
COUNTRIES_PLUS = (
%(countries_plus)s
)

# Official capitalized country names.
OFFICIAL_COUNTRIES = {
%(official_countries)s
}
'''
OFFICIAL_COUNTRIES_LINE = u'    %(code)r: %(name)r,'
COUNTRIES_LINE = u'    (%(code)r, _(%(name)r)),'
RE_VALID_LINE = re.compile(r'\s*(?P<name>.+);(?P<code>[A-Z]{2})\s*$')
RE_ACRONYM = re.compile(r'\b[A-Z](\.[A-Za-z])+\b')
RE_MC = re.compile(r'\b(Mc)(\w)')

def _cmp_value(value):
    """
    Ensure the countries are sorted correctly by replacing unicode characters
    with the basic English letter equivalent.

    """
    value = value.replace(u'\xe9', 'e').replace(u'\xf4', 'o')
    if value.startswith(u'\xc5'):
        value = 'A%s' % value[1:]
    return value


def regenerate(location='http://www.iso.org/iso/list-en1-semic-3.txt',
               filename=None, default_encoding='ISO-8859-1'):
    """
    Generate the countries Python module from a semicolon delimited file.
    
    """
    # Get the country list.
    data = urllib2.urlopen(location)
    official_countries = []
    if ('content-type' in data.headers and
                'charset=' in data.headers['content-type']):
        encoding = data.headers['content-type'].split('charset=')[-1]
    else:
        encoding = default_encoding
    for line in data:
        match = RE_VALID_LINE.match(unicode(line, encoding))
        if not match:
            continue
        country = match.groupdict()
        official_countries.append((str(country['code']), country['name']))

    # Generate template output (and the nicely titled country names).
    official_countries_lines = []
    countries_lines = []
    countries_plus = []
    for code, name in official_countries:
        country_data = {'code': code, 'name': name}
        official_countries_lines.append(OFFICIAL_COUNTRIES_LINE % country_data)
        name = string.capwords(name)
        name = RE_ACRONYM.sub(lambda match: match.group().upper(), name)
        if ', ' in name:
            important, rest = name.split(', ', 1)
            important = titlecase(important)
            # Temporarily add on a space so titlecase doesn't think that ending
            # shortwords should be titled.
            rest = titlecase('%s ' % rest)[:-1]
            plus_name = '%s %s' % (rest, important)
            plus_name = RE_MC.sub(lambda match: '%s%s' %
                                     (match.group(1), match.group(2).upper()),
                                  plus_name)
            countries_plus.append((plus_name, code))
            name = '%s, %s' % (important, rest)
        else:
            name = titlecase(name)
        name = RE_MC.sub(lambda match: '%s%s' % (match.group(1),
                                                 match.group(2).upper()), name)
        country_data['name'] = name
        countries_lines.append(COUNTRIES_LINE % country_data)
        countries_plus.append((name, code))
    # Order the countries_plus list of countries.
    countries_plus.sort(cmp=lambda a, b: cmp((_cmp_value(a[0]), a[1]),
                                             (_cmp_value(b[0]), b[1])))
    countries_plus_lines = []
    for name, code in countries_plus:
        country_data = {'code': code, 'name': name}
        countries_plus_lines.append(COUNTRIES_LINE % country_data)

    # Generate and save the file.
    if not filename:
        filename = os.path.join(os.path.dirname(os.path.dirname(
            os.path.realpath(__file__))), 'countries.py')
    # TODO: first make a backup of the file if it exists already.
    f = open(filename, 'w')
    f.write(TEMPLATE % {
        'official_countries': '\n'.join(official_countries_lines),
        'countries': '\n'.join(countries_lines),
        'countries_plus': '\n'.join(countries_plus_lines)
    })
    f.close()


if __name__ == '__main__':
    # TODO: use parseopt for location / filename
    regenerate()



from django.conf import settings


def _build_flag_url():
    if hasattr(settings, 'COUNTRIES_FLAG_URL'):
        url = settings.COUNTRIES_FLAG_URL
    else:
        url = 'flags/%(code)s.gif'
    prefix = getattr(settings, 'STATIC_URL', '') or settings.MEDIA_URL
    if not prefix.endswith('/'):
        prefix = '%s/' % prefix
    return '%s%s' % (prefix, url)


FLAG_URL = _build_flag_url()



from django.test import TestCase
from django_countries import settings


class TestCountryField(TestCase):

    def create_person(self, country='NZ'):
        from django_countries.tests import Person
        return Person.objects.create(name='Chris Beaven', country=country)

    def test_logic(self):
        person = self.create_person()

        self.assertEqual(person.country, 'NZ')
        self.assertNotEqual(person.country, 'ZZ')

        self.assert_(person.country < 'OA')
        self.assert_(person.country > 'NY')

        self.assert_(person.country)
        person.country = ''
        self.assertFalse(person.country)

    def test_unicode(self):
        person = self.create_person()
        self.assertEqual(unicode(person.country), 'NZ')

    def test_name(self):
        person = self.create_person()
        self.assertEqual(person.country.name, u'New Zealand')

    def test_flag(self):
        person = self.create_person()
        expected_url = settings.FLAG_URL % {'code': 'nz', 'code_upper': 'NZ'}
        self.assertEqual(person.country.flag, expected_url)

    def test_blank(self):
        from django_countries.tests import Person
        person = self.create_person(country=None)
        self.assertEqual(person.country, '')

        person = Person.objects.get(pk=person.pk)
        self.assertEqual(person.country, '')

    def test_len(self):
        person = self.create_person()
        self.assertEqual(len(person.country), 2)

        person = self.create_person(country=None)
        self.assertEqual(len(person.country), 0)



from django.db.models.fields import CharField
from django.utils.encoding import force_unicode, StrAndUnicode
from django_countries import settings


class Country(StrAndUnicode):
    def __init__(self, code):
        self.code = code
    
    def __unicode__(self):
        return force_unicode(self.code or u'')

    def __eq__(self, other):
        return unicode(self) == force_unicode(other)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __cmp__(self, other):
        return cmp(unicode(self), force_unicode(other))

    def __hash__(self):
        return hash(unicode(self))

    def __repr__(self):
        return "%s(code=%r)" % (self.__class__.__name__, unicode(self))

    def __nonzero__(self):
        return bool(self.code)

    def __len__(self):
        return len(unicode(self))
    
    @property
    def name(self):
        # Local import so the countries aren't loaded unless they are needed. 
        from django_countries.countries import COUNTRIES
        for code, name in COUNTRIES:
            if self.code == code:
                return name
        return ''
    
    @property
    def flag(self):
        if not self.code:
            return ''
        return settings.FLAG_URL % {'code_upper': self.code,
                                    'code': self.code.lower()}


class CountryDescriptor(object):
    """
    A descriptor for country fields on a model instance. Returns a Country when
    accessed so you can do stuff like::

        >>> instance.country.name
        u'New Zealand'
        
        >>> instance.country.flag
        '/static/flags/nz.gif'
    """
    def __init__(self, field):
        self.field = field

    def __get__(self, instance=None, owner=None):
        if instance is None:
            raise AttributeError(
                "The '%s' attribute can only be accessed from %s instances."
                % (self.field.name, owner.__name__))
        return Country(code=instance.__dict__[self.field.name])

    def __set__(self, instance, value):
        if value is not None:
            value = force_unicode(value)
        instance.__dict__[self.field.name] = value


class CountryField(CharField):
    """
    A country field for Django models that provides all ISO 3166-1 countries as
    choices.
    
    """
    descriptor_class = CountryDescriptor
 
    def __init__(self, *args, **kwargs):
        # Local import so the countries aren't loaded unless they are needed. 
        from django_countries.countries import COUNTRIES 

        kwargs.setdefault('max_length', 2) 
        kwargs.setdefault('choices', COUNTRIES) 

        super(CharField, self).__init__(*args, **kwargs) 

    def get_internal_type(self): 
        return "CharField"

    def contribute_to_class(self, cls, name):
        super(CountryField, self).contribute_to_class(cls, name)
        setattr(cls, self.name, self.descriptor_class(self))

    def get_prep_lookup(self, lookup_type, value):
        if hasattr(value, 'code'):
            value = value.code
        return super(CountryField, self).get_prep_lookup(lookup_type, value)

    def pre_save(self, *args, **kwargs):
        "Returns field's value just before saving."
        value = super(CharField, self).pre_save(*args, **kwargs)
        return self.get_prep_value(value)

    def get_prep_value(self, value):
        "Returns field's value prepared for saving into a database."
        # Convert the Country to unicode for database insertion.
        if value is None:
            return None
        return unicode(value)


# If south is installed, ensure that CountryField will be introspected just
# like a normal CharField.
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ['^django_countries\.fields\.CountryField'])
except ImportError:
    pass



from django.db import models
from django_countries import CountryField
from django_countries.tests.fields import TestCountryField


class Person(models.Model):
    name = models.CharField(max_length=50)
    country = CountryField(blank=True)



#!/usr/bin/env python
import os
from setuptools import setup, find_packages


def long_description():
    """
    Build the long description from a README file located in the same directory
    as this module.
    """
    base_path = os.path.dirname(os.path.realpath(__file__))
    readme = open(os.path.join(base_path, 'README.rst'))
    try:
        return readme.read()
    finally:
        readme.close()


setup(
    name='django-countries',
    version='1.5',
    description='Provides a country field for Django models.',
    long_description=long_description(),
    author='Chris Beaven',
    author_email='smileychris@gmail.com',
    url='http://bitbucket.org/smileychris/django-countries/',
    packages=find_packages(),
    zip_safe=False,
    package_data={'django_countries': [
        'bin/*.py',
        'static/flags/*.gif',
        'locale/*/LC_MESSAGES/*',
    ]},
    # titlecase PYPI is broken, copied the module directly for now (in /bin)
    #      requires=['titlecase'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
)



from django.utils.translation import ugettext_lazy as _

# Nicely titled (and translatable) country names.
COUNTRIES = (
    ('AF', _(u'Afghanistan')),
    ('AX', _(u'\xc5land Islands')),
    ('AL', _(u'Albania')),
    ('DZ', _(u'Algeria')),
    ('AS', _(u'American Samoa')),
    ('AD', _(u'Andorra')),
    ('AO', _(u'Angola')),
    ('AI', _(u'Anguilla')),
    ('AQ', _(u'Antarctica')),
    ('AG', _(u'Antigua and Barbuda')),
    ('AR', _(u'Argentina')),
    ('AM', _(u'Armenia')),
    ('AW', _(u'Aruba')),
    ('AU', _(u'Australia')),
    ('AT', _(u'Austria')),
    ('AZ', _(u'Azerbaijan')),
    ('BS', _(u'Bahamas')),
    ('BH', _(u'Bahrain')),
    ('BD', _(u'Bangladesh')),
    ('BB', _(u'Barbados')),
    ('BY', _(u'Belarus')),
    ('BE', _(u'Belgium')),
    ('BZ', _(u'Belize')),
    ('BJ', _(u'Benin')),
    ('BM', _(u'Bermuda')),
    ('BT', _(u'Bhutan')),
    ('BO', _(u'Bolivia, Plurinational State of')),
    ('BQ', _(u'Bonaire, Sint Eustatius and Saba')),
    ('BA', _(u'Bosnia and Herzegovina')),
    ('BW', _(u'Botswana')),
    ('BV', _(u'Bouvet Island')),
    ('BR', _(u'Brazil')),
    ('IO', _(u'British Indian Ocean Territory')),
    ('BN', _(u'Brunei Darussalam')),
    ('BG', _(u'Bulgaria')),
    ('BF', _(u'Burkina Faso')),
    ('BI', _(u'Burundi')),
    ('KH', _(u'Cambodia')),
    ('CM', _(u'Cameroon')),
    ('CA', _(u'Canada')),
    ('CV', _(u'Cape Verde')),
    ('KY', _(u'Cayman Islands')),
    ('CF', _(u'Central African Republic')),
    ('TD', _(u'Chad')),
    ('CL', _(u'Chile')),
    ('CN', _(u'China')),
    ('CX', _(u'Christmas Island')),
    ('CC', _(u'Cocos (Keeling) Islands')),
    ('CO', _(u'Colombia')),
    ('KM', _(u'Comoros')),
    ('CG', _(u'Congo')),
    ('CD', _(u'Congo, The Democratic Republic of the')),
    ('CK', _(u'Cook Islands')),
    ('CR', _(u'Costa Rica')),
    ('CI', _(u"C\xf4te D'ivoire")),
    ('HR', _(u'Croatia')),
    ('CU', _(u'Cuba')),
    ('CW', _(u'Cura\xe7ao')),
    ('CY', _(u'Cyprus')),
    ('CZ', _(u'Czech Republic')),
    ('DK', _(u'Denmark')),
    ('DJ', _(u'Djibouti')),
    ('DM', _(u'Dominica')),
    ('DO', _(u'Dominican Republic')),
    ('EC', _(u'Ecuador')),
    ('EG', _(u'Egypt')),
    ('SV', _(u'El Salvador')),
    ('GQ', _(u'Equatorial Guinea')),
    ('ER', _(u'Eritrea')),
    ('EE', _(u'Estonia')),
    ('ET', _(u'Ethiopia')),
    ('FK', _(u'Falkland Islands (Malvinas)')),
    ('FO', _(u'Faroe Islands')),
    ('FJ', _(u'Fiji')),
    ('FI', _(u'Finland')),
    ('FR', _(u'France')),
    ('GF', _(u'French Guiana')),
    ('PF', _(u'French Polynesia')),
    ('TF', _(u'French Southern Territories')),
    ('GA', _(u'Gabon')),
    ('GM', _(u'Gambia')),
    ('GE', _(u'Georgia')),
    ('DE', _(u'Germany')),
    ('GH', _(u'Ghana')),
    ('GI', _(u'Gibraltar')),
    ('GR', _(u'Greece')),
    ('GL', _(u'Greenland')),
    ('GD', _(u'Grenada')),
    ('GP', _(u'Guadeloupe')),
    ('GU', _(u'Guam')),
    ('GT', _(u'Guatemala')),
    ('GG', _(u'Guernsey')),
    ('GN', _(u'Guinea')),
    ('GW', _(u'Guinea-bissau')),
    ('GY', _(u'Guyana')),
    ('HT', _(u'Haiti')),
    ('HM', _(u'Heard Island and McDonald Islands')),
    ('VA', _(u'Holy See (Vatican City State)')),
    ('HN', _(u'Honduras')),
    ('HK', _(u'Hong Kong')),
    ('HU', _(u'Hungary')),
    ('IS', _(u'Iceland')),
    ('IN', _(u'India')),
    ('ID', _(u'Indonesia')),
    ('IR', _(u'Iran, Islamic Republic of')),
    ('IQ', _(u'Iraq')),
    ('IE', _(u'Ireland')),
    ('IM', _(u'Isle of Man')),
    ('IL', _(u'Israel')),
    ('IT', _(u'Italy')),
    ('JM', _(u'Jamaica')),
    ('JP', _(u'Japan')),
    ('JE', _(u'Jersey')),
    ('JO', _(u'Jordan')),
    ('KZ', _(u'Kazakhstan')),
    ('KE', _(u'Kenya')),
    ('KI', _(u'Kiribati')),
    ('KP', _(u"Korea, Democratic People's Republic of")),
    ('KR', _(u'Korea, Republic of')),
    ('KW', _(u'Kuwait')),
    ('KG', _(u'Kyrgyzstan')),
    ('LA', _(u"Lao People's Democratic Republic")),
    ('LV', _(u'Latvia')),
    ('LB', _(u'Lebanon')),
    ('LS', _(u'Lesotho')),
    ('LR', _(u'Liberia')),
    ('LY', _(u'Libya')),
    ('LI', _(u'Liechtenstein')),
    ('LT', _(u'Lithuania')),
    ('LU', _(u'Luxembourg')),
    ('MO', _(u'Macao')),
    ('MK', _(u'Macedonia, The Former Yugoslav Republic of')),
    ('MG', _(u'Madagascar')),
    ('MW', _(u'Malawi')),
    ('MY', _(u'Malaysia')),
    ('MV', _(u'Maldives')),
    ('ML', _(u'Mali')),
    ('MT', _(u'Malta')),
    ('MH', _(u'Marshall Islands')),
    ('MQ', _(u'Martinique')),
    ('MR', _(u'Mauritania')),
    ('MU', _(u'Mauritius')),
    ('YT', _(u'Mayotte')),
    ('MX', _(u'Mexico')),
    ('FM', _(u'Micronesia, Federated States of')),
    ('MD', _(u'Moldova, Republic of')),
    ('MC', _(u'Monaco')),
    ('MN', _(u'Mongolia')),
    ('ME', _(u'Montenegro')),
    ('MS', _(u'Montserrat')),
    ('MA', _(u'Morocco')),
    ('MZ', _(u'Mozambique')),
    ('MM', _(u'Myanmar')),
    ('NA', _(u'Namibia')),
    ('NR', _(u'Nauru')),
    ('NP', _(u'Nepal')),
    ('NL', _(u'Netherlands')),
    ('NC', _(u'New Caledonia')),
    ('NZ', _(u'New Zealand')),
    ('NI', _(u'Nicaragua')),
    ('NE', _(u'Niger')),
    ('NG', _(u'Nigeria')),
    ('NU', _(u'Niue')),
    ('NF', _(u'Norfolk Island')),
    ('MP', _(u'Northern Mariana Islands')),
    ('NO', _(u'Norway')),
    ('OM', _(u'Oman')),
    ('PK', _(u'Pakistan')),
    ('PW', _(u'Palau')),
    ('PS', _(u'Palestinian Territory, Occupied')),
    ('PA', _(u'Panama')),
    ('PG', _(u'Papua New Guinea')),
    ('PY', _(u'Paraguay')),
    ('PE', _(u'Peru')),
    ('PH', _(u'Philippines')),
    ('PN', _(u'Pitcairn')),
    ('PL', _(u'Poland')),
    ('PT', _(u'Portugal')),
    ('PR', _(u'Puerto Rico')),
    ('QA', _(u'Qatar')),
    ('RE', _(u'R\xe9union')),
    ('RO', _(u'Romania')),
    ('RU', _(u'Russian Federation')),
    ('RW', _(u'Rwanda')),
    ('BL', _(u'Saint Barth\xe9lemy')),
    ('SH', _(u'Saint Helena, Ascension and Tristan Da Cunha')),
    ('KN', _(u'Saint Kitts and Nevis')),
    ('LC', _(u'Saint Lucia')),
    ('MF', _(u'Saint Martin (French Part)')),
    ('PM', _(u'Saint Pierre and Miquelon')),
    ('VC', _(u'Saint Vincent and the Grenadines')),
    ('WS', _(u'Samoa')),
    ('SM', _(u'San Marino')),
    ('ST', _(u'Sao Tome and Principe')),
    ('SA', _(u'Saudi Arabia')),
    ('SN', _(u'Senegal')),
    ('RS', _(u'Serbia')),
    ('SC', _(u'Seychelles')),
    ('SL', _(u'Sierra Leone')),
    ('SG', _(u'Singapore')),
    ('SX', _(u'Sint Maarten (Dutch Part)')),
    ('SK', _(u'Slovakia')),
    ('SI', _(u'Slovenia')),
    ('SB', _(u'Solomon Islands')),
    ('SO', _(u'Somalia')),
    ('ZA', _(u'South Africa')),
    ('GS', _(u'South Georgia and the South Sandwich Islands')),
    ('SS', _(u'South Sudan')),
    ('ES', _(u'Spain')),
    ('LK', _(u'Sri Lanka')),
    ('SD', _(u'Sudan')),
    ('SR', _(u'Suriname')),
    ('SJ', _(u'Svalbard and Jan Mayen')),
    ('SZ', _(u'Swaziland')),
    ('SE', _(u'Sweden')),
    ('CH', _(u'Switzerland')),
    ('SY', _(u'Syrian Arab Republic')),
    ('TW', _(u'Taiwan, Province of China')),
    ('TJ', _(u'Tajikistan')),
    ('TZ', _(u'Tanzania, United Republic of')),
    ('TH', _(u'Thailand')),
    ('TL', _(u'Timor-leste')),
    ('TG', _(u'Togo')),
    ('TK', _(u'Tokelau')),
    ('TO', _(u'Tonga')),
    ('TT', _(u'Trinidad and Tobago')),
    ('TN', _(u'Tunisia')),
    ('TR', _(u'Turkey')),
    ('TM', _(u'Turkmenistan')),
    ('TC', _(u'Turks and Caicos Islands')),
    ('TV', _(u'Tuvalu')),
    ('UG', _(u'Uganda')),
    ('UA', _(u'Ukraine')),
    ('AE', _(u'United Arab Emirates')),
    ('GB', _(u'United Kingdom')),
    ('US', _(u'United States')),
    ('UM', _(u'United States Minor Outlying Islands')),
    ('UY', _(u'Uruguay')),
    ('UZ', _(u'Uzbekistan')),
    ('VU', _(u'Vanuatu')),
    ('VE', _(u'Venezuela, Bolivarian Republic of')),
    ('VN', _(u'Viet Nam')),
    ('VG', _(u'Virgin Islands, British')),
    ('VI', _(u'Virgin Islands, U.S.')),
    ('WF', _(u'Wallis and Futuna')),
    ('EH', _(u'Western Sahara')),
    ('YE', _(u'Yemen')),
    ('ZM', _(u'Zambia')),
    ('ZW', _(u'Zimbabwe')),
)

# Nicely titled country names with duplicates for those which contain a comma
# (containing the non-comma'd version).
COUNTRIES_PLUS = (
    ('AF', _(u'Afghanistan')),
    ('AX', _(u'\xc5land Islands')),
    ('AL', _(u'Albania')),
    ('DZ', _(u'Algeria')),
    ('AS', _(u'American Samoa')),
    ('AD', _(u'Andorra')),
    ('AO', _(u'Angola')),
    ('AI', _(u'Anguilla')),
    ('AQ', _(u'Antarctica')),
    ('AG', _(u'Antigua and Barbuda')),
    ('AR', _(u'Argentina')),
    ('AM', _(u'Armenia')),
    ('AW', _(u'Aruba')),
    ('SH', _(u'Ascension and Tristan Da Cunha Saint Helena')),
    ('AU', _(u'Australia')),
    ('AT', _(u'Austria')),
    ('AZ', _(u'Azerbaijan')),
    ('BS', _(u'Bahamas')),
    ('BH', _(u'Bahrain')),
    ('BD', _(u'Bangladesh')),
    ('BB', _(u'Barbados')),
    ('BY', _(u'Belarus')),
    ('BE', _(u'Belgium')),
    ('BZ', _(u'Belize')),
    ('BJ', _(u'Benin')),
    ('BM', _(u'Bermuda')),
    ('BT', _(u'Bhutan')),
    ('VE', _(u'Bolivarian Republic of Venezuela')),
    ('BO', _(u'Bolivia, Plurinational State of')),
    ('BQ', _(u'Bonaire, Sint Eustatius and Saba')),
    ('BA', _(u'Bosnia and Herzegovina')),
    ('BW', _(u'Botswana')),
    ('BV', _(u'Bouvet Island')),
    ('BR', _(u'Brazil')),
    ('IO', _(u'British Indian Ocean Territory')),
    ('VG', _(u'British Virgin Islands')),
    ('BN', _(u'Brunei Darussalam')),
    ('BG', _(u'Bulgaria')),
    ('BF', _(u'Burkina Faso')),
    ('BI', _(u'Burundi')),
    ('KH', _(u'Cambodia')),
    ('CM', _(u'Cameroon')),
    ('CA', _(u'Canada')),
    ('CV', _(u'Cape Verde')),
    ('KY', _(u'Cayman Islands')),
    ('CF', _(u'Central African Republic')),
    ('TD', _(u'Chad')),
    ('CL', _(u'Chile')),
    ('CN', _(u'China')),
    ('CX', _(u'Christmas Island')),
    ('CC', _(u'Cocos (Keeling) Islands')),
    ('CO', _(u'Colombia')),
    ('KM', _(u'Comoros')),
    ('CG', _(u'Congo')),
    ('CD', _(u'Congo, The Democratic Republic of the')),
    ('CK', _(u'Cook Islands')),
    ('CR', _(u'Costa Rica')),
    ('CI', _(u"C\xf4te D'ivoire")),
    ('HR', _(u'Croatia')),
    ('CU', _(u'Cuba')),
    ('CW', _(u'Cura\xe7ao')),
    ('CY', _(u'Cyprus')),
    ('CZ', _(u'Czech Republic')),
    ('KP', _(u"Democratic People's Republic of Korea")),
    ('DK', _(u'Denmark')),
    ('DJ', _(u'Djibouti')),
    ('DM', _(u'Dominica')),
    ('DO', _(u'Dominican Republic')),
    ('EC', _(u'Ecuador')),
    ('EG', _(u'Egypt')),
    ('SV', _(u'El Salvador')),
    ('GQ', _(u'Equatorial Guinea')),
    ('ER', _(u'Eritrea')),
    ('EE', _(u'Estonia')),
    ('ET', _(u'Ethiopia')),
    ('FK', _(u'Falkland Islands (Malvinas)')),
    ('FO', _(u'Faroe Islands')),
    ('FM', _(u'Federated States of Micronesia')),
    ('FJ', _(u'Fiji')),
    ('FI', _(u'Finland')),
    ('FR', _(u'France')),
    ('GF', _(u'French Guiana')),
    ('PF', _(u'French Polynesia')),
    ('TF', _(u'French Southern Territories')),
    ('GA', _(u'Gabon')),
    ('GM', _(u'Gambia')),
    ('GE', _(u'Georgia')),
    ('DE', _(u'Germany')),
    ('GH', _(u'Ghana')),
    ('GI', _(u'Gibraltar')),
    ('GR', _(u'Greece')),
    ('GL', _(u'Greenland')),
    ('GD', _(u'Grenada')),
    ('GP', _(u'Guadeloupe')),
    ('GU', _(u'Guam')),
    ('GT', _(u'Guatemala')),
    ('GG', _(u'Guernsey')),
    ('GN', _(u'Guinea')),
    ('GW', _(u'Guinea-bissau')),
    ('GY', _(u'Guyana')),
    ('HT', _(u'Haiti')),
    ('HM', _(u'Heard Island and McDonald Islands')),
    ('VA', _(u'Holy See (Vatican City State)')),
    ('HN', _(u'Honduras')),
    ('HK', _(u'Hong Kong')),
    ('HU', _(u'Hungary')),
    ('IS', _(u'Iceland')),
    ('IN', _(u'India')),
    ('ID', _(u'Indonesia')),
    ('IR', _(u'Iran, Islamic Republic of')),
    ('IQ', _(u'Iraq')),
    ('IE', _(u'Ireland')),
    ('IR', _(u'Islamic Republic of Iran')),
    ('IM', _(u'Isle of Man')),
    ('IL', _(u'Israel')),
    ('IT', _(u'Italy')),
    ('JM', _(u'Jamaica')),
    ('JP', _(u'Japan')),
    ('JE', _(u'Jersey')),
    ('JO', _(u'Jordan')),
    ('KZ', _(u'Kazakhstan')),
    ('KE', _(u'Kenya')),
    ('KI', _(u'Kiribati')),
    ('KP', _(u"Korea, Democratic People's Republic of")),
    ('KR', _(u'Korea, Republic of')),
    ('KW', _(u'Kuwait')),
    ('KG', _(u'Kyrgyzstan')),
    ('LA', _(u"Lao People's Democratic Republic")),
    ('LV', _(u'Latvia')),
    ('LB', _(u'Lebanon')),
    ('LS', _(u'Lesotho')),
    ('LR', _(u'Liberia')),
    ('LY', _(u'Libya')),
    ('LI', _(u'Liechtenstein')),
    ('LT', _(u'Lithuania')),
    ('LU', _(u'Luxembourg')),
    ('MO', _(u'Macao')),
    ('MK', _(u'Macedonia, The Former Yugoslav Republic of')),
    ('MG', _(u'Madagascar')),
    ('MW', _(u'Malawi')),
    ('MY', _(u'Malaysia')),
    ('MV', _(u'Maldives')),
    ('ML', _(u'Mali')),
    ('MT', _(u'Malta')),
    ('MH', _(u'Marshall Islands')),
    ('MQ', _(u'Martinique')),
    ('MR', _(u'Mauritania')),
    ('MU', _(u'Mauritius')),
    ('YT', _(u'Mayotte')),
    ('MX', _(u'Mexico')),
    ('FM', _(u'Micronesia, Federated States of')),
    ('MD', _(u'Moldova, Republic of')),
    ('MC', _(u'Monaco')),
    ('MN', _(u'Mongolia')),
    ('ME', _(u'Montenegro')),
    ('MS', _(u'Montserrat')),
    ('MA', _(u'Morocco')),
    ('MZ', _(u'Mozambique')),
    ('MM', _(u'Myanmar')),
    ('NA', _(u'Namibia')),
    ('NR', _(u'Nauru')),
    ('NP', _(u'Nepal')),
    ('NL', _(u'Netherlands')),
    ('NC', _(u'New Caledonia')),
    ('NZ', _(u'New Zealand')),
    ('NI', _(u'Nicaragua')),
    ('NE', _(u'Niger')),
    ('NG', _(u'Nigeria')),
    ('NU', _(u'Niue')),
    ('NF', _(u'Norfolk Island')),
    ('MP', _(u'Northern Mariana Islands')),
    ('NO', _(u'Norway')),
    ('PS', _(u'Occupied Palestinian Territory')),
    ('OM', _(u'Oman')),
    ('PK', _(u'Pakistan')),
    ('PW', _(u'Palau')),
    ('PS', _(u'Palestinian Territory, Occupied')),
    ('PA', _(u'Panama')),
    ('PG', _(u'Papua New Guinea')),
    ('PY', _(u'Paraguay')),
    ('PE', _(u'Peru')),
    ('PH', _(u'Philippines')),
    ('PN', _(u'Pitcairn')),
    ('BO', _(u'Plurinational State of Bolivia')),
    ('PL', _(u'Poland')),
    ('PT', _(u'Portugal')),
    ('TW', _(u'Province of China Taiwan')),
    ('PR', _(u'Puerto Rico')),
    ('QA', _(u'Qatar')),
    ('KR', _(u'Republic of Korea')),
    ('MD', _(u'Republic of Moldova')),
    ('RE', _(u'R\xe9union')),
    ('RO', _(u'Romania')),
    ('RU', _(u'Russian Federation')),
    ('RW', _(u'Rwanda')),
    ('BL', _(u'Saint Barth\xe9lemy')),
    ('SH', _(u'Saint Helena, Ascension and Tristan Da Cunha')),
    ('KN', _(u'Saint Kitts and Nevis')),
    ('LC', _(u'Saint Lucia')),
    ('MF', _(u'Saint Martin (French Part)')),
    ('PM', _(u'Saint Pierre and Miquelon')),
    ('VC', _(u'Saint Vincent and the Grenadines')),
    ('WS', _(u'Samoa')),
    ('SM', _(u'San Marino')),
    ('ST', _(u'Sao Tome and Principe')),
    ('SA', _(u'Saudi Arabia')),
    ('SN', _(u'Senegal')),
    ('RS', _(u'Serbia')),
    ('SC', _(u'Seychelles')),
    ('SL', _(u'Sierra Leone')),
    ('SG', _(u'Singapore')),
    ('BQ', _(u'Sint Eustatius and Saba Bonaire')),
    ('SX', _(u'Sint Maarten (Dutch Part)')),
    ('SK', _(u'Slovakia')),
    ('SI', _(u'Slovenia')),
    ('SB', _(u'Solomon Islands')),
    ('SO', _(u'Somalia')),
    ('ZA', _(u'South Africa')),
    ('GS', _(u'South Georgia and the South Sandwich Islands')),
    ('SS', _(u'South Sudan')),
    ('ES', _(u'Spain')),
    ('LK', _(u'Sri Lanka')),
    ('SD', _(u'Sudan')),
    ('SR', _(u'Suriname')),
    ('SJ', _(u'Svalbard and Jan Mayen')),
    ('SZ', _(u'Swaziland')),
    ('SE', _(u'Sweden')),
    ('CH', _(u'Switzerland')),
    ('SY', _(u'Syrian Arab Republic')),
    ('TW', _(u'Taiwan, Province of China')),
    ('TJ', _(u'Tajikistan')),
    ('TZ', _(u'Tanzania, United Republic of')),
    ('TH', _(u'Thailand')),
    ('CD', _(u'The Democratic Republic of the Congo')),
    ('MK', _(u'The Former Yugoslav Republic of Macedonia')),
    ('TL', _(u'Timor-leste')),
    ('TG', _(u'Togo')),
    ('TK', _(u'Tokelau')),
    ('TO', _(u'Tonga')),
    ('TT', _(u'Trinidad and Tobago')),
    ('TN', _(u'Tunisia')),
    ('TR', _(u'Turkey')),
    ('TM', _(u'Turkmenistan')),
    ('TC', _(u'Turks and Caicos Islands')),
    ('TV', _(u'Tuvalu')),
    ('VI', _(u'U.S. Virgin Islands')),
    ('UG', _(u'Uganda')),
    ('UA', _(u'Ukraine')),
    ('AE', _(u'United Arab Emirates')),
    ('GB', _(u'United Kingdom')),
    ('TZ', _(u'United Republic of Tanzania')),
    ('US', _(u'United States')),
    ('UM', _(u'United States Minor Outlying Islands')),
    ('UY', _(u'Uruguay')),
    ('UZ', _(u'Uzbekistan')),
    ('VU', _(u'Vanuatu')),
    ('VE', _(u'Venezuela, Bolivarian Republic of')),
    ('VN', _(u'Viet Nam')),
    ('VG', _(u'Virgin Islands, British')),
    ('VI', _(u'Virgin Islands, U.S.')),
    ('WF', _(u'Wallis and Futuna')),
    ('EH', _(u'Western Sahara')),
    ('YE', _(u'Yemen')),
    ('ZM', _(u'Zambia')),
    ('ZW', _(u'Zimbabwe')),
)

# Official capitalized country names.
OFFICIAL_COUNTRIES = {
    'AF': u'AFGHANISTAN',
    'AX': u'\xc5LAND ISLANDS',
    'AL': u'ALBANIA',
    'DZ': u'ALGERIA',
    'AS': u'AMERICAN SAMOA',
    'AD': u'ANDORRA',
    'AO': u'ANGOLA',
    'AI': u'ANGUILLA',
    'AQ': u'ANTARCTICA',
    'AG': u'ANTIGUA AND BARBUDA',
    'AR': u'ARGENTINA',
    'AM': u'ARMENIA',
    'AW': u'ARUBA',
    'AU': u'AUSTRALIA',
    'AT': u'AUSTRIA',
    'AZ': u'AZERBAIJAN',
    'BS': u'BAHAMAS',
    'BH': u'BAHRAIN',
    'BD': u'BANGLADESH',
    'BB': u'BARBADOS',
    'BY': u'BELARUS',
    'BE': u'BELGIUM',
    'BZ': u'BELIZE',
    'BJ': u'BENIN',
    'BM': u'BERMUDA',
    'BT': u'BHUTAN',
    'BO': u'BOLIVIA, PLURINATIONAL STATE OF',
    'BQ': u'BONAIRE, SINT EUSTATIUS AND SABA',
    'BA': u'BOSNIA AND HERZEGOVINA',
    'BW': u'BOTSWANA',
    'BV': u'BOUVET ISLAND',
    'BR': u'BRAZIL',
    'IO': u'BRITISH INDIAN OCEAN TERRITORY',
    'BN': u'BRUNEI DARUSSALAM',
    'BG': u'BULGARIA',
    'BF': u'BURKINA FASO',
    'BI': u'BURUNDI',
    'KH': u'CAMBODIA',
    'CM': u'CAMEROON',
    'CA': u'CANADA',
    'CV': u'CAPE VERDE',
    'KY': u'CAYMAN ISLANDS',
    'CF': u'CENTRAL AFRICAN REPUBLIC',
    'TD': u'CHAD',
    'CL': u'CHILE',
    'CN': u'CHINA',
    'CX': u'CHRISTMAS ISLAND',
    'CC': u'COCOS (KEELING) ISLANDS',
    'CO': u'COLOMBIA',
    'KM': u'COMOROS',
    'CG': u'CONGO',
    'CD': u'CONGO, THE DEMOCRATIC REPUBLIC OF THE',
    'CK': u'COOK ISLANDS',
    'CR': u'COSTA RICA',
    'CI': u"C\xd4TE D'IVOIRE",
    'HR': u'CROATIA',
    'CU': u'CUBA',
    'CW': u'CURA\xc7AO',
    'CY': u'CYPRUS',
    'CZ': u'CZECH REPUBLIC',
    'DK': u'DENMARK',
    'DJ': u'DJIBOUTI',
    'DM': u'DOMINICA',
    'DO': u'DOMINICAN REPUBLIC',
    'EC': u'ECUADOR',
    'EG': u'EGYPT',
    'SV': u'EL SALVADOR',
    'GQ': u'EQUATORIAL GUINEA',
    'ER': u'ERITREA',
    'EE': u'ESTONIA',
    'ET': u'ETHIOPIA',
    'FK': u'FALKLAND ISLANDS (MALVINAS)',
    'FO': u'FAROE ISLANDS',
    'FJ': u'FIJI',
    'FI': u'FINLAND',
    'FR': u'FRANCE',
    'GF': u'FRENCH GUIANA',
    'PF': u'FRENCH POLYNESIA',
    'TF': u'FRENCH SOUTHERN TERRITORIES',
    'GA': u'GABON',
    'GM': u'GAMBIA',
    'GE': u'GEORGIA',
    'DE': u'GERMANY',
    'GH': u'GHANA',
    'GI': u'GIBRALTAR',
    'GR': u'GREECE',
    'GL': u'GREENLAND',
    'GD': u'GRENADA',
    'GP': u'GUADELOUPE',
    'GU': u'GUAM',
    'GT': u'GUATEMALA',
    'GG': u'GUERNSEY',
    'GN': u'GUINEA',
    'GW': u'GUINEA-BISSAU',
    'GY': u'GUYANA',
    'HT': u'HAITI',
    'HM': u'HEARD ISLAND AND MCDONALD ISLANDS',
    'VA': u'HOLY SEE (VATICAN CITY STATE)',
    'HN': u'HONDURAS',
    'HK': u'HONG KONG',
    'HU': u'HUNGARY',
    'IS': u'ICELAND',
    'IN': u'INDIA',
    'ID': u'INDONESIA',
    'IR': u'IRAN, ISLAMIC REPUBLIC OF',
    'IQ': u'IRAQ',
    'IE': u'IRELAND',
    'IM': u'ISLE OF MAN',
    'IL': u'ISRAEL',
    'IT': u'ITALY',
    'JM': u'JAMAICA',
    'JP': u'JAPAN',
    'JE': u'JERSEY',
    'JO': u'JORDAN',
    'KZ': u'KAZAKHSTAN',
    'KE': u'KENYA',
    'KI': u'KIRIBATI',
    'KP': u"KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF",
    'KR': u'KOREA, REPUBLIC OF',
    'KW': u'KUWAIT',
    'KG': u'KYRGYZSTAN',
    'LA': u"LAO PEOPLE'S DEMOCRATIC REPUBLIC",
    'LV': u'LATVIA',
    'LB': u'LEBANON',
    'LS': u'LESOTHO',
    'LR': u'LIBERIA',
    'LY': u'LIBYA',
    'LI': u'LIECHTENSTEIN',
    'LT': u'LITHUANIA',
    'LU': u'LUXEMBOURG',
    'MO': u'MACAO',
    'MK': u'MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF',
    'MG': u'MADAGASCAR',
    'MW': u'MALAWI',
    'MY': u'MALAYSIA',
    'MV': u'MALDIVES',
    'ML': u'MALI',
    'MT': u'MALTA',
    'MH': u'MARSHALL ISLANDS',
    'MQ': u'MARTINIQUE',
    'MR': u'MAURITANIA',
    'MU': u'MAURITIUS',
    'YT': u'MAYOTTE',
    'MX': u'MEXICO',
    'FM': u'MICRONESIA, FEDERATED STATES OF',
    'MD': u'MOLDOVA, REPUBLIC OF',
    'MC': u'MONACO',
    'MN': u'MONGOLIA',
    'ME': u'MONTENEGRO',
    'MS': u'MONTSERRAT',
    'MA': u'MOROCCO',
    'MZ': u'MOZAMBIQUE',
    'MM': u'MYANMAR',
    'NA': u'NAMIBIA',
    'NR': u'NAURU',
    'NP': u'NEPAL',
    'NL': u'NETHERLANDS',
    'NC': u'NEW CALEDONIA',
    'NZ': u'NEW ZEALAND',
    'NI': u'NICARAGUA',
    'NE': u'NIGER',
    'NG': u'NIGERIA',
    'NU': u'NIUE',
    'NF': u'NORFOLK ISLAND',
    'MP': u'NORTHERN MARIANA ISLANDS',
    'NO': u'NORWAY',
    'OM': u'OMAN',
    'PK': u'PAKISTAN',
    'PW': u'PALAU',
    'PS': u'PALESTINIAN TERRITORY, OCCUPIED',
    'PA': u'PANAMA',
    'PG': u'PAPUA NEW GUINEA',
    'PY': u'PARAGUAY',
    'PE': u'PERU',
    'PH': u'PHILIPPINES',
    'PN': u'PITCAIRN',
    'PL': u'POLAND',
    'PT': u'PORTUGAL',
    'PR': u'PUERTO RICO',
    'QA': u'QATAR',
    'RE': u'R\xc9UNION',
    'RO': u'ROMANIA',
    'RU': u'RUSSIAN FEDERATION',
    'RW': u'RWANDA',
    'BL': u'SAINT BARTH\xc9LEMY',
    'SH': u'SAINT HELENA, ASCENSION AND TRISTAN DA CUNHA',
    'KN': u'SAINT KITTS AND NEVIS',
    'LC': u'SAINT LUCIA',
    'MF': u'SAINT MARTIN (FRENCH PART)',
    'PM': u'SAINT PIERRE AND MIQUELON',
    'VC': u'SAINT VINCENT AND THE GRENADINES',
    'WS': u'SAMOA',
    'SM': u'SAN MARINO',
    'ST': u'SAO TOME AND PRINCIPE',
    'SA': u'SAUDI ARABIA',
    'SN': u'SENEGAL',
    'RS': u'SERBIA',
    'SC': u'SEYCHELLES',
    'SL': u'SIERRA LEONE',
    'SG': u'SINGAPORE',
    'SX': u'SINT MAARTEN (DUTCH PART)',
    'SK': u'SLOVAKIA',
    'SI': u'SLOVENIA',
    'SB': u'SOLOMON ISLANDS',
    'SO': u'SOMALIA',
    'ZA': u'SOUTH AFRICA',
    'GS': u'SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS',
    'SS': u'SOUTH SUDAN',
    'ES': u'SPAIN',
    'LK': u'SRI LANKA',
    'SD': u'SUDAN',
    'SR': u'SURINAME',
    'SJ': u'SVALBARD AND JAN MAYEN',
    'SZ': u'SWAZILAND',
    'SE': u'SWEDEN',
    'CH': u'SWITZERLAND',
    'SY': u'SYRIAN ARAB REPUBLIC',
    'TW': u'TAIWAN, PROVINCE OF CHINA',
    'TJ': u'TAJIKISTAN',
    'TZ': u'TANZANIA, UNITED REPUBLIC OF',
    'TH': u'THAILAND',
    'TL': u'TIMOR-LESTE',
    'TG': u'TOGO',
    'TK': u'TOKELAU',
    'TO': u'TONGA',
    'TT': u'TRINIDAD AND TOBAGO',
    'TN': u'TUNISIA',
    'TR': u'TURKEY',
    'TM': u'TURKMENISTAN',
    'TC': u'TURKS AND CAICOS ISLANDS',
    'TV': u'TUVALU',
    'UG': u'UGANDA',
    'UA': u'UKRAINE',
    'AE': u'UNITED ARAB EMIRATES',
    'GB': u'UNITED KINGDOM',
    'US': u'UNITED STATES',
    'UM': u'UNITED STATES MINOR OUTLYING ISLANDS',
    'UY': u'URUGUAY',
    'UZ': u'UZBEKISTAN',
    'VU': u'VANUATU',
    'VE': u'VENEZUELA, BOLIVARIAN REPUBLIC OF',
    'VN': u'VIET NAM',
    'VG': u'VIRGIN ISLANDS, BRITISH',
    'VI': u'VIRGIN ISLANDS, U.S.',
    'WF': u'WALLIS AND FUTUNA',
    'EH': u'WESTERN SAHARA',
    'YE': u'YEMEN',
    'ZM': u'ZAMBIA',
    'ZW': u'ZIMBABWE',
}
